# Generated by Django 1.11.12 on 2018-07-31 16:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0013_drop_insurance'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subjectregistration',
            name='variable_symbol',
            field=models.BigIntegerField(db_index=True, editable=False, null=True, verbose_name='variable symbol'),
        ),
    ]
