{% extends "admin/change_list.html" %}
{% load i18n l10n admin_urls %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ opts.app_config.verbose_name }}</a>
&rsaquo; <a href="{% url message_opts|admin_urlname:'changelist' %}">{{ message_opts.verbose_name_plural|capfirst|escape }}</a>
&rsaquo; {{ title }}
</div>
{% endblock %}
